package args

import (
	"errors"
	"flag"
	"os"
	"strings"

	"golang.org/x/oauth2"
)

type Arguments struct {
	DatabasePath    string
	TemplatePath    string
	StaticPath      string
	CloudflareToken string
	CloudflareZone  string
	Port            int
	Server          bool
	Migrate         bool
	Scheduler       bool

	OauthConfig      *oauth2.Config
	OauthUserInfoURI string
}

func GetArgs() (*Arguments, error) {
	port := flag.Int("port", 8080, "Port to listen on")
	databasePath := flag.String("database-path", "./hatecomputers.db", "Path to the SQLite database")
	templatePath := flag.String("template-path", "./templates", "Path to the template directory")
	staticPath := flag.String("static-path", "./static", "Path to the static directory")

	scheduler := flag.Bool("scheduler", false, "Run scheduled jobs via cron")
	server := flag.Bool("server", false, "Run the server")
	migrate := flag.Bool("migrate", false, "Run the migrations")

	flag.Parse()

	cloudflareToken := os.Getenv("CLOUDFLARE_TOKEN")
	cloudflareZone := os.Getenv("CLOUDFLARE_ZONE")

	oauthClientID := os.Getenv("OAUTH_CLIENT_ID")
	oauthClientSecret := os.Getenv("OAUTH_CLIENT_SECRET")
	oauthScopes := os.Getenv("OAUTH_SCOPES")
	oauthAuthURL := os.Getenv("OAUTH_AUTH_URL")
	oauthTokenURL := os.Getenv("OAUTH_TOKEN_URL")
	oauthRedirectURI := os.Getenv("OAUTH_REDIRECT_URI")
	oauthUserInfoURI := os.Getenv("OAUTH_USER_INFO_URI")

	envVars := [][]string{
		{cloudflareToken, "CLOUDFLARE_TOKEN"},
		{cloudflareZone, "CLOUDFLARE_ZONE"},
		{oauthClientID, "OAUTH_CLIENT_ID"},
		{oauthClientSecret, "OAUTH_CLIENT_SECRET"},
		{oauthScopes, "OAUTH_SCOPES"},
		{oauthAuthURL, "OAUTH_AUTH_URL"},
		{oauthTokenURL, "OAUTH_TOKEN_URL"},
		{oauthRedirectURI, "OAUTH_REDIRECT_URI"},
		{oauthUserInfoURI, "OAUTH_USER_INFO_URI"},
	}

	for _, envVar := range envVars {
		if envVar[0] == "" {
			return nil, errors.New("please set the " + envVar[1] + " environment variable")
		}
	}

	oauthConfig := &oauth2.Config{
		ClientID:     oauthClientID,
		ClientSecret: oauthClientSecret,
		Scopes:       strings.Split(oauthScopes, ","),
		Endpoint: oauth2.Endpoint{
			AuthURL:  oauthAuthURL,
			TokenURL: oauthTokenURL,
		},
		RedirectURL: oauthRedirectURI,
	}

	arguments := &Arguments{
		DatabasePath:    *databasePath,
		TemplatePath:    *templatePath,
		StaticPath:      *staticPath,
		CloudflareToken: cloudflareToken,
		CloudflareZone:  cloudflareZone,
		Port:            *port,
		Server:          *server,
		Migrate:         *migrate,
		Scheduler:       *scheduler,

		OauthConfig:      oauthConfig,
		OauthUserInfoURI: oauthUserInfoURI,
	}

	return arguments, nil
}
